fl

{
132x168LCD(LQ020B8UB02)
PropForth5.5

2015/05/12 11:55:08

}

\ =========================================================================== 
\ Constants 
\ =========================================================================== 
0 wconstant serial
1 wconstant clk
2 wconstant res
3 wconstant lcd_wr
4 wconstant A0

\ =========================================================================== 
\ Variables 
\ =========================================================================== 

variable buffer d60 allot     \ [2bytes(16bits) x 32bytes]
variable mask                 \ mask(32bit)
wvariable dot_mask
h8000 wconstant char_start
hBFFF wconstant char_end
variable tmp
wvariable char_addr
wvariable x                   \ 0 < x < 8    132/16=8.25  column/1char=16bits [2bytes/1pixel]
wvariable y                   \ 0 < y < 5    168/32=5.25  page/1char=32lines
wvariable buff_ptr
wvariable color_ptr
wvariable disp_color
variable color d16 allot      \ color-table 20bytes [1color:2bytes]

\ =========================================================================== 
\ Main 
\ =========================================================================== 

\ Convert 8bits-parallel to 8bit-serial by shift-register
\ ( n1 n2 n3 -- ) n1:8bits-data  n2:serial  n3:clk 
lockdict create a_shift_data forthentry
$C_a_lxasm w, h127  h113  1- tuck - h9 lshift or here W@ alignl h10 lshift or l,
z2WyPW1 l, zfiPZB l, z1SyLI[ l, z2WyPb1 l, zfiPeB l, z1SyLI[ l, zfyPOO l, z2WyPj8 l,
zgyPO1 l, z1jixZD l, z1[ixZC l, 0 l, 0 l, z1bixZC l, z3[yPnR l, 0 l,
0 l, z1[ixZC l, z1SyLI[ l, z1SV01X l,
freedict

\ Set serial pin to hi
\ ( -- )
: serial_hi serial pinhi ;
\ Set serial pin to lo
\ ( -- )
: serial_lo serial pinlo ;

\ Issue clock pulse
\ ( -- )
: clk_out clk pinhi clk pinlo ;

\ Reset 132x168LCD(LQ020B8UB02)
\ ( -- )
: reset res pinlo 1 delms res pinhi ;

\ Set lcd_wr to hi
\ ( -- )
: lcd_wr_hi lcd_wr pinhi ;
\ Set lcd_wr to lo
\ ( -- )
: lcd_wr_lo lcd_wr pinlo ;

\ Set A0 to hi
\ ( -- )
: A0_hi A0 pinhi ;
\ Set A0 to lo
\ ( -- )
: A0_lo A0 pinlo ;


\ Write command to 132x168LCD(LQ020B8UB02)
\ ( n1 -- ) n1:data[8bitd]
: command
A0_lo                    \ A0=L
lcd_wr_lo                \ lcd_wr=L
\ shift_data
serial clk a_shift_data
lcd_wr_hi                \ lcd_wr=H
;

\ Write parameter to 132x168LCD(LQ020B8UB02)
\ ( n1 -- ) n1:parameter[8bitd]
: parameter
A0_hi                    \ A0=H
lcd_wr_lo                \ lcd_wr=L
\ shift_data
serial clk a_shift_data
lcd_wr_hi                \ lcd_wr=H
;

\ Write 2byte in vram
\ ( n1 n2 -- )   n1:lower byte n2:upprt byte 
: vram_wr
A0_hi                    \ A0=H
lcd_wr_lo                \ lcd_wr=L
serial clk a_shift_data
lcd_wr_hi
lcd_wr_lo
serial clk a_shift_data
lcd_wr_hi                \ lcd_wr=H
;

\ Initialize 132x168LCD(LQ020B8UB02)
: lcd_initial
serial 5 0 do dup pinout 1+ loop drop        \ Set P0-P4 to output
res pinhi lcd_wr pinhi
reset                                        \ Reset LCD
                                         
1 delms
hCA command                                  \ Timing Control
hC parameter                                 \ Parameter:Mode1 Panel scan rate=49Hz
3 parameter
0 parameter
hD parameter
3 parameter
h28 parameter
0 parameter
                    
d30 delms
h94 command                                  \ Sleep Disable
d200 delms
\ Write RAM under RB mode
h5C command                                  
\ Clear vram (132x162x2bytes)  [R:G:B]=[5bits:6bits:5bits]
d21384 0 do 0 0 vram_wr loop     
hAF command                                  \ Display On
;

\ Power off LCD
\ ( -- )
: LCD_off
hAE command
h95 command
d500 delms
;

\ Get 16bits from 1line(32bits)
\ ( n1 -- n2 )  n1:32bit data  n2:16bit data
: get_line        
0 swap                             \ ( 0 n1 )
d16 0 do 
     dup mask L@ and 0<>           \ Check mask-bit 
     if swap 1+ swap then          \ ( 1 n1 )
     mask dup L@ 2 lshift swap L!  \ Shift mask to left 2bits
\     i d16 < if swap 1 lshift swap then
     swap 1 lshift swap            \ ( 1<<1 n1 )
loop
drop
;

\ Copy 1-line[16bits] to buffer
\ ( n1 -- ) n1:16bits
: buffer_copy     
buff_ptr W@              \ ( n1 [buff_ptr] ) 
swap over W!             \ ( n1 )
2+ buff_ptr W!           \ ( -- )
;

\ Read 1-line of ROM-font from buffer
\ ( -- n1) n1:16bits
: buffer_read     
buff_ptr W@ W@           \ Get [buff_ptr] 
\ Update buff_ptr
buff_ptr dup W@ 2+ swap W!
;

\ Set x to column
\ ( n1 -- )  n1:column  1column=[2bytes(/1pixel) x16pixel]
: column_set       
d32 u*                   \ start column address (byte number) 
dup tmp L! 
d31 +                    \ end column address (byte number)
dup 8 rshift             \ end column addres(upper byte)
swap hFF and             \ end column addres(lower byte)
tmp L@ dup 8 rshift      \ start column address(upper byte)
swap hFF and             \ start column address(lower byte)
\ set column
h15 command parameter parameter parameter parameter       
;

\ Set y to page
\ ( n1 -- )  n1:page   1page=[32lines]
: page_set          
d32 u*                   \ start page address 
dup d31 +                \ end page address
swap
\ set page
h75 command parameter parameter       
;

\
\ ( n1 -- )  n1:font-addr
: disp_pair        \ font-addr disp_pair
char_addr W!
\ odd and even character
2 0 do 
     \ Get 1-character[16bits x 32lines]
     buffer buff_ptr W!
     i 1+ tmp L!
     char_addr W@
     d32 0 do 
          tmp L@ mask L! 
          dup L@              \ Get 32bits
          get_line            \ Get 1-line[16bits] in character 
          buffer_copy         \ Copy 1disp-data[16bits] to buffer
          4+                  \ next line
          loop
     drop
     
     buffer buff_ptr W!       \ Set buffer to buff_ptr       
     x W@ column_set          \ Set column
     y W@ page_set            \ Set page
     \ Set color and check if color exceed color table
     color_ptr W@ dup W@ disp_color W! 2+ dup color d20 + > 
     if drop color then
     color_ptr W! 
     \ Write 1-character to vram inside LCD         
     h5C command
     d32 0 do                               \ page direction
          h8000 dot_mask W!                 \ 16bits      
          buffer_read                       \ Read data(1-line) from buffer
          d16 0 do                          \ column derection
               dup 
               dot_mask W@ and 
               if 
                    disp_color dup W@ 
                    8 rshift vram_wr
               else 
                    0 dup 
                    vram_wr
               then
               dot_mask dup W@ 
               1 rshift swap W!              \ Shift dot_mask to right 1bit (next dot)
          loop
          drop
     loop
     y W@ x W@       
     1+ dup 8 =                              \ check x>7
     if 
          drop 0                             \ x=0 
          swap 1+ dup 5 =                    \ Check y>4
          if drop 0 then                     \ y=0
          swap
     then
     x W! y W!                               \ Store (x,y)        
loop
;

\ Add 2 to address and copy
\ ( n1 -- n1+2 n1+2)
: 2+dup 2+ dup ;
\ Set value to color-table 'color'
\ ( -- )
: color_list
color dup
h1F swap W!     \ blue
2+dup
hF800 swap W!   \ red
2+dup
hF81F swap W!   \ magenda
2+dup
h7E0 swap W!    \ green
2+dup
h7FE swap W!    \ cyan
2+dup
hFFE0 swap W!   \ yellow
2+dup
hFFFF swap W!   \ white
2+dup
hA18A swap W!   \ brown
2+dup
hFBC0 swap W!   \ orange
2+dup
hF998 swap W!   \ pink
drop
;

\ Display ROM font
\ ( -- )
: demo
lcd_initial              \ Initialize LCD
                 
color_list               \ set color list
color color_ptr W!       \ Set color pointor
0 dup y W! x W!          \ Set (x,y)
char_start               \ Start address of ROM font
\ d256 0 do
begin
     dup 
     disp_pair
     h80 +               \ next font pair 
     dup char_end >
     \ Set Start address if character end 
     if drop char_start then
     fkey? swap drop   
until
drop  
;

{
\ Convert 8bits-parallel to 8bit-serial by shift-register
\ ( n1 n2 n3 -- ) n1:8bits-data  n2:serial  n3:clk 
\ $C_treg1 -- clkm
\ $C_treg2 -- serialm  
\ $C_treg3 -- loop counter

fl
build_BootOpt :rasm
     \ get clkm
     mov       $C_treg1 , # 1         
     shl       $C_treg1 , $C_stTOS
     spop
     \ get serialm
     mov       $C_treg2 , # 1     
     shl       $C_treg2 , $C_stTOS
     spop
     
     \ shift data to left 24bits
     shl       $C_stTOS , # d24
     \ set loop-count 
     mov       $C_treg3 , # 8
__1
     shl       $C_stTOS , # 1 wc
     \ set serial to Hi/Lo
     muxc      outa , $C_treg2 
     \ clock pulse Lo:500nsec  Hi:200nsec
     andn      outa , $C_treg1
     nop
     nop
          \ set _latch to Hi    
     or       outa , $C_treg1
     
     djnz      $C_treg3 , # __1
     nop
     nop
     andn      outa , $C_treg1
         
     spop
     jexit
;asm a_shift_data
}
     
